@@ -610,9 +610,13 @@ check_orsf_inputs <- function(data = NULL,
                               n_tree = NULL,
                               n_split = NULL,
                               n_retry = NULL,
+                              n_thread = NULL,
                               mtry = NULL,
+                              sample_with_replacement = NULL,
+                              sample_fraction = NULL,
                               leaf_min_events = NULL,
                               leaf_min_obs = NULL,
+                              split_rule = NULL,
                               split_min_events = NULL,
                               split_min_obs = NULL,
                               split_min_stat = NULL,
@@ -621,7 +625,8 @@ check_orsf_inputs <- function(data = NULL,
                               oobag_eval_every = NULL,
                               importance = NULL,
                               tree_seeds = NULL,
-                              attach_data = NULL){
+                              attach_data = NULL,
+                              verbose_progress = NULL){
 
  if(!is.null(data)){
 
@@ -796,6 +801,25 @@ check_orsf_inputs <- function(data = NULL,
 
  }
 
+ if(!is.null(n_thread)){
+
+  check_arg_type(arg_value = n_thread,
+                 arg_name = 'n_thread',
+                 expected_type = 'numeric')
+
+  check_arg_is_integer(arg_name = 'n_thread',
+                       arg_value = n_thread)
+
+  check_arg_gteq(arg_name = 'n_thread',
+                 arg_value = n_thread,
+                 bound = 0)
+
+  check_arg_length(arg_name = 'n_thread',
+                   arg_value = n_thread,
+                   expected_length = 1)
+
+ }
+
  if(!is.null(mtry)){
 
   check_arg_type(arg_value = mtry,
@@ -815,6 +839,38 @@ check_orsf_inputs <- function(data = NULL,
 
  }
 
+ if(!is.null(sample_with_replacement)){
+
+  check_arg_type(arg_value = sample_with_replacement,
+                 arg_name = 'sample_with_replacement',
+                 expected_type = 'logical')
+
+  check_arg_length(arg_name = 'sample_with_replacement',
+                   arg_value = sample_with_replacement,
+                   expected_length = 1)
+
+ }
+
+ if(!is.null(sample_fraction)){
+
+  check_arg_type(arg_value = sample_fraction,
+                 arg_name = 'sample_fraction',
+                 expected_type = 'numeric')
+
+  check_arg_gt(arg_value = sample_fraction,
+               arg_name = 'sample_fraction',
+               bound = 0)
+
+  check_arg_lteq(arg_value = sample_fraction,
+                 arg_name = 'sample_fraction',
+                 bound = 1)
+
+  check_arg_length(arg_value = sample_fraction,
+                   arg_name = 'sample_fraction',
+                   expected_length = 1)
+
+ }
+
  if(!is.null(leaf_min_events)){
 
   check_arg_type(arg_value = leaf_min_events,
@@ -852,6 +908,22 @@ check_orsf_inputs <- function(data = NULL,
 
  }
 
+ if(!is.null(split_rule)){
+
+  check_arg_type(arg_value = split_rule,
+                 arg_name = 'split_rule',
+                 expected_type = 'character')
+
+  check_arg_length(arg_value = split_rule,
+                   arg_name = 'split_rule',
+                   expected_length = 1)
+
+  check_arg_is_valid(arg_value = split_rule,
+                     arg_name = 'split_rule',
+                     valid_options = c("logrank", "cstat"))
+
+ }
+
  if(!is.null(split_min_events)){
 
   check_arg_type(arg_value = split_min_events,
@@ -916,19 +988,14 @@ check_orsf_inputs <- function(data = NULL,
                    arg_name = 'oobag_pred_type',
                    expected_length = 1)
 
-  if(oobag_pred_type == 'mort') stop(
-   "Out-of-bag mortality predictions aren't supported yet. ",
-   " Sorry for the inconvenience - we plan on including this option",
-   " in a future update.",
-   call. = FALSE
-  )
-
   check_arg_is_valid(arg_value = oobag_pred_type,
                      arg_name = 'oobag_pred_type',
                      valid_options = c("none",
                                        "surv",
                                        "risk",
-                                       "chf"))
+                                       "chf",
+                                       "mort",
+                                       "leaf"))
 
  }
 
@@ -938,13 +1005,17 @@ check_orsf_inputs <- function(data = NULL,
                  arg_name = 'oobag_pred_horizon',
                  expected_type = 'numeric')
 
-  check_arg_length(arg_value = oobag_pred_horizon,
-                   arg_name = 'oobag_pred_horizon',
-                   expected_length = 1)
+  # check_arg_length(arg_value = oobag_pred_horizon,
+  #                  arg_name = 'oobag_pred_horizon',
+  #                  expected_length = 1)
 
-  check_arg_gteq(arg_value = oobag_pred_horizon,
-                 arg_name = 'oobag_pred_horizon',
-                 bound = 0)
+  for(i in seq_along(oobag_pred_horizon)){
+
+   check_arg_gteq(arg_value = oobag_pred_horizon[i],
+                  arg_name = 'oobag_pred_horizon',
+                  bound = 0)
+
+  }
 
  }
 
@@ -1000,7 +1071,7 @@ check_orsf_inputs <- function(data = NULL,
 
   check_arg_is_integer(tree_seeds, arg_name = 'tree_seeds')
 
-  if(length(tree_seeds) != n_tree){
+  if(length(tree_seeds) > 1 && length(tree_seeds) != n_tree){
 
    stop('tree_seeds should have length <', n_tree,
         "> (the number of trees) but instead has length <",
@@ -1022,6 +1093,18 @@ check_orsf_inputs <- function(data = NULL,
 
  }
 
+ if(!is.null(verbose_progress)){
+
+  check_arg_type(arg_value = verbose_progress,
+                 arg_name = 'verbose_progress',
+                 expected_type = 'logical')
+
+  check_arg_length(arg_value = verbose_progress,
+                   arg_name = 'verbose_progress',
+                   expected_length = 1)
+
+ }
+
 }
 
 #' Check inputs for orsf_pd()
@@ -1204,7 +1287,8 @@ check_pd_inputs <- function(object,
                new_data = new_data,
                pred_horizon = pred_horizon,
                pred_type = pred_type,
-               na_action = na_action)
+               na_action = na_action,
+               valid_pred_types = c("risk", "surv", "chf", "mort"))
 
 }
 
@@ -1481,7 +1565,8 @@ check_predict <- function(object,
                           pred_horizon = NULL,
                           pred_type = NULL,
                           na_action = NULL,
-                          boundary_checks = NULL){
+                          boundary_checks = TRUE,
+                          valid_pred_types = c("risk", "surv", "chf", "mort", "leaf")){
 
  if(!is.null(new_data)){
 
@@ -1547,10 +1632,7 @@ check_predict <- function(object,
 
   check_arg_is_valid(arg_value = pred_type,
                      arg_name = 'pred_type',
-                     valid_options = c("risk",
-                                       "surv",
-                                       "chf",
-                                       "mort"))
+                     valid_options = valid_pred_types)
 
  }
 
@@ -1615,8 +1697,8 @@ check_oobag_fun <- function(oobag_fun){
 
  oobag_fun_args <- names(formals(oobag_fun))
 
- if(length(oobag_fun_args) != 2) stop(
-  "oobag_fun should have 2 input arguments but instead has ",
+ if(length(oobag_fun_args) != 3) stop(
+  "oobag_fun should have 3 input arguments but instead has ",
   length(oobag_fun_args),
   call. = FALSE
  )
@@ -1627,8 +1709,14 @@ check_oobag_fun <- function(oobag_fun){
   call. = FALSE
  )
 
- if(oobag_fun_args[2] != 's_vec') stop(
-  "the second input argument of oobag_fun should be named 's_vec' ",
+ if(oobag_fun_args[2] != 'w_vec') stop(
+  "the second input argument of oobag_fun should be named 'w_vec' ",
+  "but is instead named '", oobag_fun_args[1], "'",
+  call. = FALSE
+ )
+
+ if(oobag_fun_args[3] != 's_vec') stop(
+  "the third input argument of oobag_fun should be named 's_vec' ",
   "but is instead named '", oobag_fun_args[2], "'",
   call. = FALSE
  )
@@ -1637,9 +1725,12 @@ check_oobag_fun <- function(oobag_fun){
  test_status <- rep(c(0,1), each = 50)
 
  .y_mat <- cbind(time = test_time, status = test_status)
+ .w_vec <- rep(1, times = 100)
  .s_vec <- seq(0.9, 0.1, length.out = 100)
 
- test_output <- try(oobag_fun(y_mat = .y_mat, s_vec = .s_vec),
+ test_output <- try(oobag_fun(y_mat = .y_mat,
+                              w_vec = .w_vec,
+                              s_vec = .s_vec),
                     silent = FALSE)
 
  if(is_error(test_output)){
@@ -1649,8 +1740,9 @@ check_oobag_fun <- function(oobag_fun){
        "test_time <- seq(from = 1, to = 5, length.out = 100)\n",
        "test_status <- rep(c(0,1), each = 50)\n\n",
        "y_mat <- cbind(time = test_time, status = test_status)\n",
+       "w_vec <- rep(1, times = 100)\n",
        "s_vec <- seq(0.9, 0.1, length.out = 100)\n\n",
-       "test_output <- oobag_fun(y_mat = y_mat, s_vec = s_vec)\n\n",
+       "test_output <- oobag_fun(y_mat = y_mat, w_vec = w_vec, s_vec = s_vec)\n\n",
        "test_output should be a numeric value of length 1",
        call. = FALSE)
 
