@@ -12,15 +12,24 @@
 #'
 #' @noRd
 
-infer_pred_horizon <- function(object, pred_horizon){
+infer_pred_horizon <- function(object, pred_type, pred_horizon){
 
  check_arg_is(object, 'object', 'orsf_fit')
 
+ if(pred_type %in% c("mort", "leaf")){
+  # value of pred_horizon does not matter for these types of prediction
+  pred_horizon <- 1
+ }
+
+ # see if it was previously specified
  if(is.null(pred_horizon)) pred_horizon <- object$pred_horizon
 
- if(is.null(pred_horizon))
+ # throw error if pred_type requires pred_horizon
+ if(is.null(pred_horizon)){
   stop("pred_horizon was not specified and could not be found in object.",
        call. = FALSE)
+ }
+
 
  pred_horizon
 
