@@ -60,6 +60,8 @@
 #'  percentile in the object's training data. If `FALSE`, these checks are
 #'  skipped.
 #'
+#' @param n_thread `r roxy_n_thread_header("computing predictions")`
+#'
 #' @param ... `r roxy_dots()`
 #'
 #' @return a [data.table][data.table::data.table-package] containing
@@ -86,6 +88,7 @@ orsf_pd_oob <- function(object,
                         prob_values = c(0.025, 0.50, 0.975),
                         prob_labels = c('lwr', 'medn', 'upr'),
                         boundary_checks = TRUE,
+                        n_thread = 1,
                         ...){
 
  check_dots(list(...), orsf_pd_oob)
@@ -99,6 +102,7 @@ orsf_pd_oob <- function(object,
                       prob_values = prob_values,
                       prob_labels = prob_labels,
                       boundary_checks = boundary_checks,
+                      n_thread = n_thread,
                       oobag = TRUE,
                       type_output = 'smry')
 
@@ -114,6 +118,7 @@ orsf_pd_inb <- function(object,
                         prob_values = c(0.025, 0.50, 0.975),
                         prob_labels = c('lwr', 'medn', 'upr'),
                         boundary_checks = TRUE,
+                        n_thread = 1,
                         ...){
 
  check_dots(list(...), orsf_pd_inb)
@@ -132,6 +137,7 @@ orsf_pd_inb <- function(object,
                       prob_values = prob_values,
                       prob_labels = prob_labels,
                       boundary_checks = boundary_checks,
+                      n_thread = n_thread,
                       oobag = FALSE,
                       type_output = 'smry')
 
@@ -149,6 +155,7 @@ orsf_pd_new <- function(object,
                         prob_values = c(0.025, 0.50, 0.975),
                         prob_labels = c('lwr', 'medn', 'upr'),
                         boundary_checks = TRUE,
+                        n_thread = 1,
                         ...){
 
  check_dots(list(...), orsf_pd_new)
@@ -163,6 +170,7 @@ orsf_pd_new <- function(object,
                       prob_values = prob_values,
                       prob_labels = prob_labels,
                       boundary_checks = boundary_checks,
+                      n_thread = n_thread,
                       oobag = FALSE,
                       type_output = 'smry')
 
@@ -192,6 +200,7 @@ orsf_ice_oob <- function(object,
                          pred_type = 'risk',
                          expand_grid = TRUE,
                          boundary_checks = TRUE,
+                         n_thread = 1,
                          ...){
 
  check_dots(list(...), orsf_ice_oob)
@@ -203,6 +212,7 @@ orsf_ice_oob <- function(object,
                       pred_type = pred_type,
                       expand_grid = expand_grid,
                       boundary_checks = boundary_checks,
+                      n_thread = n_thread,
                       oobag = TRUE,
                       type_output = 'ice')
 
@@ -216,6 +226,7 @@ orsf_ice_inb <- function(object,
                          pred_type = 'risk',
                          expand_grid = TRUE,
                          boundary_checks = TRUE,
+                         n_thread = 1,
                          ...){
 
  check_dots(list(...), orsf_ice_oob)
@@ -232,6 +243,7 @@ orsf_ice_inb <- function(object,
                       pred_type = pred_type,
                       expand_grid = expand_grid,
                       boundary_checks = boundary_checks,
+                      n_thread = n_thread,
                       oobag = FALSE,
                       type_output = 'ice')
 
@@ -247,6 +259,7 @@ orsf_ice_new <- function(object,
                          na_action = 'fail',
                          expand_grid = TRUE,
                          boundary_checks = TRUE,
+                         n_thread = 1,
                          ...){
 
  check_dots(list(...), orsf_ice_new)
@@ -259,6 +272,7 @@ orsf_ice_new <- function(object,
                       na_action = na_action,
                       expand_grid = expand_grid,
                       boundary_checks = boundary_checks,
+                      n_thread = n_thread,
                       oobag = FALSE,
                       type_output = 'ice')
 
@@ -290,11 +304,15 @@ orsf_pred_dependence <- function(object,
                                  expand_grid,
                                  prob_values = NULL,
                                  prob_labels = NULL,
+                                 boundary_checks,
+                                 n_thread,
                                  oobag,
-                                 type_output,
-                                 boundary_checks){
+                                 type_output){
 
- pred_horizon <- infer_pred_horizon(object, pred_horizon)
+ pred_horizon <- infer_pred_horizon(object, pred_type, pred_horizon)
+
+ # make a visible binding for CRAN
+ id_variable = NULL
 
  if(is.null(prob_values)) prob_values <- c(0.025, 0.50, 0.975)
  if(is.null(prob_labels)) prob_labels <- c('lwr', 'medn', 'upr')
@@ -304,19 +322,13 @@ orsf_pred_dependence <- function(object,
                  expand_grid     = expand_grid,
                  prob_values     = prob_values,
                  prob_labels     = prob_labels,
+                 oobag           = oobag,
                  boundary_checks = boundary_checks,
                  new_data        = pd_data,
                  pred_type       = pred_type,
                  pred_horizon    = pred_horizon,
                  na_action       = na_action)
 
- if(pred_type == 'mort') stop(
-  "mortality predictions aren't supported in partial dependence functions",
-  " yet. Sorry for the inconvenience - we plan on including this option",
-  " in a future update.",
-  call. = FALSE
- )
-
  if(oobag && is.null(object$data))
   stop("no data were found in object. ",
        "did you use attach_data = FALSE when ",
@@ -329,8 +341,6 @@ orsf_pred_dependence <- function(object,
        pred_type, " predictions.", call. = FALSE)
  }
 
- type_input <- if(expand_grid) 'grid' else 'loop'
-
  names_x_data <- intersect(get_names_x(object), names(pd_data))
 
  cc <- which(stats::complete.cases(select_cols(pd_data, names_x_data)))
@@ -345,7 +355,6 @@ orsf_pred_dependence <- function(object,
 
  x_new <- prep_x_from_orsf(object, data = pd_data[cc, ])
 
-
  # the values in pred_spec need to be centered & scaled to match x_new,
  # which is also centered and scaled
  means <- get_means(object)
@@ -355,56 +364,202 @@ orsf_pred_dependence <- function(object,
   pred_spec[[i]] <- (pred_spec[[i]] - means[i]) / standard_deviations[i]
  }
 
- if(is.data.frame(pred_spec)) type_input <- 'grid'
+ pred_type_R <- switch(pred_type,
+                       "risk" = 1,
+                       "surv" = 2,
+                       "chf"  = 3,
+                       "mort" = 4)
 
+ fi <- get_fctr_info(object)
 
- pd_fun_structure <- switch(type_input,
-                            'grid' = pd_grid,
-                            'loop' = pd_loop)
+ if(expand_grid){
 
- pd_fun_predict <- switch(paste(oobag, type_output, sep = "_"),
-                          "TRUE_ice" = pd_oob_ice,
-                          "TRUE_smry" = pd_oob_smry,
-                          "FALSE_ice" = pd_new_ice,
-                          "FALSE_smry" = pd_new_smry)
+  if(!is.data.frame(pred_spec))
+   pred_spec <- expand.grid(pred_spec, stringsAsFactors = TRUE)
 
- pred_type_cpp <- switch(
-  pred_type,
-  "risk" = "R",
-  "surv" = "S",
-  "chf"  = "H",
-  "mort" = "M"
- )
+  for(i in seq_along(fi$cols)){
 
- out_list <- lapply(
+   ii <- fi$cols[i]
 
-  X = pred_horizon,
+   if(is.character(pred_spec[[ii]]) && !fi$ordr[i]){
 
-  FUN = function(.pred_horizon){
+    pred_spec[[ii]] <- factor(pred_spec[[ii]], levels = fi$lvls[[ii]])
 
-   pd_fun_structure(object,
-                    x_new,
-                    pred_spec,
-                    .pred_horizon,
-                    pd_fun_predict,
-                    type_output,
-                    prob_values,
-                    prob_labels,
-                    oobag,
-                    pred_type_cpp)
+   }
 
   }
 
- )
+  check_new_data_fctrs(new_data  = pred_spec,
+                       names_x   = get_names_x(object),
+                       fi_ref    = fi,
+                       label_new = "pred_spec")
 
- names(out_list) <- as.character(pred_horizon)
+  pred_spec_new <- ref_code(x_data = pred_spec,
+                            fi = get_fctr_info(object),
+                            names_x_data = names(pred_spec))
 
- out <- rbindlist(l = out_list,
-                  fill = TRUE,
-                  idcol = 'pred_horizon')
+  x_cols <- list(match(names(pred_spec_new), colnames(x_new)) - 1)
+
+  pred_spec_new <- list(as.matrix(pred_spec_new))
+
+  pd_bind <- list(pred_spec)
+
+ } else {
+
+  pred_spec_new <- pd_bind <- x_cols <- list()
+
+  for(i in seq_along(pred_spec)){
+
+   pred_spec_new[[i]]  <- as.data.frame(pred_spec[i])
+   pd_name <- names(pred_spec)[i]
+
+   pd_bind[[i]] <- data.frame(
+    variable = pd_name,
+    value = rep(NA_real_, length(pred_spec[[i]])),
+    level = rep(NA_character_, length(pred_spec[[i]]))
+   )
+
+   if(pd_name %in% fi$cols) {
+
+    pd_bind[[i]]$level <- as.character(pred_spec[[i]])
+
+    pred_spec_new[[i]] <- ref_code(pred_spec_new[[i]],
+                                   fi = fi,
+                                   names_x_data = pd_name)
+
+   } else {
+
+    pd_bind[[i]]$value <- pred_spec[[i]]
+
+   }
+
+   x_cols[[i]] <- match(names(pred_spec_new[[i]]), colnames(x_new))-1
+   pred_spec_new[[i]] <- as.matrix(pred_spec_new[[i]])
+
+  }
+
+ }
+
+ orsf_out <- orsf_cpp(x = x_new,
+                      y = matrix(1, ncol=2),
+                      w = rep(1, nrow(x_new)),
+                      tree_type_R = get_tree_type(object),
+                      tree_seeds = get_tree_seeds(object),
+                      loaded_forest = object$forest,
+                      n_tree = get_n_tree(object),
+                      mtry = get_mtry(object),
+                      sample_with_replacement = get_sample_with_replacement(object),
+                      sample_fraction = get_sample_fraction(object),
+                      vi_type_R = 0,
+                      vi_max_pvalue = get_vi_max_pvalue(object),
+                      lincomb_R_function = get_f_beta(object),
+                      oobag_R_function = get_f_oobag_eval(object),
+                      leaf_min_events = get_leaf_min_events(object),
+                      leaf_min_obs = get_leaf_min_obs(object),
+                      split_rule_R = switch(get_split_rule(object),
+                                            "logrank" = 1,
+                                            "cstat" = 2),
+                      split_min_events = get_split_min_events(object),
+                      split_min_obs = get_split_min_obs(object),
+                      split_min_stat = get_split_min_stat(object),
+                      split_max_cuts = get_n_split(object),
+                      split_max_retry = get_n_retry(object),
+                      lincomb_type_R = switch(get_orsf_type(object),
+                                              'fast' = 1,
+                                              'cph' = 1,
+                                              'random' = 2,
+                                              'net' = 3,
+                                              'custom' = 4),
+                      lincomb_eps = get_cph_eps(object),
+                      lincomb_iter_max = get_cph_iter_max(object),
+                      lincomb_scale = get_cph_do_scale(object),
+                      lincomb_alpha = get_net_alpha(object),
+                      lincomb_df_target = get_net_df_target(object),
+                      lincomb_ties_method = switch(
+                       tolower(get_cph_method(object)),
+                       'breslow' = 0,
+                       'efron'   = 1
+                      ),
+                      pred_type_R = pred_type_R,
+                      pred_mode = FALSE,
+                      pred_aggregate = TRUE,
+                      pred_horizon = pred_horizon,
+                      oobag = oobag,
+                      oobag_eval_type_R = 0,
+                      oobag_eval_every = get_n_tree(object),
+                      pd_type_R = switch(type_output,
+                                         "smry" = 1L,
+                                         "ice" = 2L),
+                      pd_x_vals = pred_spec_new,
+                      pd_x_cols = x_cols,
+                      pd_probs = prob_values,
+                      n_thread = n_thread,
+                      write_forest = FALSE,
+                      run_forest = TRUE,
+                      verbosity = 0)
+
+ pd_vals <- orsf_out$pd_values
+
+ for(i in seq_along(pd_vals)){
+
+  pd_bind[[i]]$id_variable <- seq(nrow(pd_bind[[i]]))
+
+  for(j in seq_along(pd_vals[[i]])){
+
+   pd_vals[[i]][[j]] <- matrix(pd_vals[[i]][[j]],
+                               nrow=length(pred_horizon),
+                               byrow = T)
+
+   rownames(pd_vals[[i]][[j]]) <- pred_horizon
+
+   if(type_output=='smry')
+    colnames(pd_vals[[i]][[j]]) <- c('mean', prob_labels)
+   else
+    colnames(pd_vals[[i]][[j]]) <- c(paste(1:nrow(x_new)))
+
+   pd_vals[[i]][[j]] <- as.data.table(pd_vals[[i]][[j]],
+                                      keep.rownames = 'pred_horizon')
+
+   if(type_output == 'ice')
+    pd_vals[[i]][[j]] <- melt(data = pd_vals[[i]][[j]],
+                              id.vars = 'pred_horizon',
+                              variable.name = 'id_row',
+                              value.name = 'pred')
+
+  }
+
+  pd_vals[[i]] <- rbindlist(pd_vals[[i]], idcol = 'id_variable')
+
+  pd_vals[[i]] <- merge(pd_vals[[i]],
+                        as.data.table(pd_bind[[i]]),
+                        by = 'id_variable')
+
+ }
+
+
+ out <- rbindlist(pd_vals)
+
+ # missings may occur when oobag=TRUE and n_tree is small
+ if(type_output == 'ice') out <- collapse::na_omit(out, cols = 'pred')
+
+ ids <- c('id_variable', if(type_output == 'ice') 'id_row')
+
+ mid <- setdiff(names(out), c(ids, 'mean', prob_labels, 'pred'))
+
+ end <- setdiff(names(out), c(ids, mid))
+
+ setcolorder(out, neworder = c(ids, mid, end))
 
  out[, pred_horizon := as.numeric(pred_horizon)]
 
+ # not needed for summary
+ if(type_output == 'smry')
+  out[, id_variable := NULL]
+
+ # not needed for mort
+ if(pred_type == 'mort')
+  out[, pred_horizon := NULL]
+
  # put data back into original scale
  for(j in intersect(names(means), names(pred_spec))){
 
@@ -434,193 +589,3 @@ orsf_pred_dependence <- function(object,
 }
 
 
-#' grid working function in orsf_pd family
-#'
-#' This function expands pred_spec into a grid with all combos of inputs,
-#'   and computes partial dependence for each one.
-#'
-#' @inheritParams orsf_pred_dependence
-#' @param x_new the x-matrix used to compute partial dependence
-#' @param pd_fun_predict which cpp function to use.
-#'
-#' @return a `data.table` containing summarized partial dependence
-#'   values if using `orsf_pd_summery` or individual conditional
-#'   expectation (ICE) partial dependence if using `orsf_ice`.
-#'
-#' @noRd
-
-pd_grid <- function(object,
-                    x_new,
-                    pred_spec,
-                    pred_horizon,
-                    pd_fun_predict,
-                    type_output,
-                    prob_values,
-                    prob_labels,
-                    oobag,
-                    pred_type_cpp){
-
- if(!is.data.frame(pred_spec))
-  pred_spec <- expand.grid(pred_spec, stringsAsFactors = TRUE)
-
- fi_ref <- get_fctr_info(object)
-
- for(i in seq_along(fi_ref$cols)){
-
-  ii <- fi_ref$cols[i]
-
-  if(is.character(pred_spec[[ii]]) && !fi_ref$ordr[i]){
-
-   pred_spec[[ii]] <- factor(pred_spec[[ii]],
-                             levels = fi_ref$lvls[[ii]])
-
-  }
-
- }
-
- check_new_data_fctrs(new_data  = pred_spec,
-                      names_x   = get_names_x(object),
-                      fi_ref    = fi_ref,
-                      label_new = "pred_spec")
-
- pred_spec_new <- ref_code(x_data = pred_spec,
-                           fi = get_fctr_info(object),
-                           names_x_data = names(pred_spec))
-
- x_cols <- match(names(pred_spec_new), colnames(x_new))
-
- pd_vals <- pd_fun_predict(forest      = object$forest,
-                           x_new_      = x_new,
-                           x_cols_     = x_cols-1,
-                           x_vals_     = as_matrix(pred_spec_new),
-                           probs_      = prob_values,
-                           time_dbl    = pred_horizon,
-                           pred_type   = pred_type_cpp)
-
-
- if(type_output == 'smry'){
-
-  rownames(pd_vals) <- c('mean', prob_labels)
-  output <- cbind(pred_spec, t(pd_vals))
-  .names <- names(output)
-
- }
-
- if(type_output == 'ice'){
-
-  colnames(pd_vals) <- c('id_variable', 'pred')
-  pred_spec$id_variable <- seq(nrow(pred_spec))
-  output <- merge(pred_spec, pd_vals, by = 'id_variable')
-  output$id_row <- rep(seq(nrow(x_new)), pred_horizon = nrow(pred_spec))
-
-  ids <- c('id_variable', 'id_row')
-  .names <- c(ids, setdiff(names(output), ids))
-
- }
-
- as.data.table(output[, .names])
-
-}
-
-#' loop working function in orsf_pd family
-#'
-#' This function loops through the items in pred_spec one by one,
-#'   computing partial dependence for each one separately.
-#'
-#' @inheritParams orsf_pd_
-#' @param x_new the x-matrix used to compute partial dependence
-#' @param pd_fun_predict which cpp function to use.
-#'
-#' @return a `data.table` containing summarized partial dependence
-#'   values if using `orsf_pd_summery` or individual conditional
-#'   expectation (ICE) partial dependence if using `orsf_ice`.
-#'
-#' @noRd
-
-pd_loop <- function(object,
-                    x_new,
-                    pred_spec,
-                    pred_horizon,
-                    pd_fun_predict,
-                    type_output,
-                    prob_values,
-                    prob_labels,
-                    oobag,
-                    pred_type_cpp){
-
- fi <- get_fctr_info(object)
-
- output <- vector(mode = 'list', length = length(pred_spec))
-
- for(i in seq_along(pred_spec)){
-
-  pd_new  <- as.data.frame(pred_spec[i])
-  pd_name <- names(pred_spec)[i]
-
-  pd_bind <- data.frame(variable = pd_name,
-                        value = rep(NA_real_, length(pred_spec[[i]])),
-                        level = rep(NA_character_, length(pred_spec[[i]])))
-
-  if(pd_name %in% fi$cols) {
-
-   pd_bind$level <- as.character(pred_spec[[i]])
-
-   pd_new <- ref_code(pd_new,
-                      fi = fi,
-                      names_x_data = pd_name)
-  } else {
-
-   pd_bind$value <- pred_spec[[i]]
-
-  }
-
-  x_cols <- match(names(pd_new), colnames(x_new))
-
-  x_vals <- x_new[, x_cols]
-
-
-  pd_vals <- pd_fun_predict(forest      = object$forest,
-                            x_new_      = x_new,
-                            x_cols_     = x_cols-1,
-                            x_vals_     = as.matrix(pd_new),
-                            probs_      = prob_values,
-                            time_dbl    = pred_horizon,
-                            pred_type   = pred_type_cpp)
-
-
-  # pd_fun_predict modifies x_new by reference, so reset it.
-  x_new[, x_cols] <- x_vals
-
-  if(type_output == 'smry'){
-
-   rownames(pd_vals) <- c('mean', prob_labels)
-   output[[i]] <- cbind(pd_bind, t(pd_vals))
-
-  }
-
-  if(type_output == 'ice'){
-
-   colnames(pd_vals) <- c('id_variable', 'pred')
-   pd_bind$id_variable <- seq(nrow(pd_bind))
-   output[[i]] <- merge(pd_bind, pd_vals, by = 'id_variable')
-   output[[i]]$id_row <- seq(nrow(output[[i]]))
-
-  }
-
- }
-
- output <- rbindlist(output)
-
- if(type_output == 'ice'){
-
-  ids <- c('id_variable', 'id_row')
-  .names <- c(ids, setdiff(names(output), ids))
-  setcolorder(output, neworder = .names)
-
- }
-
-
- output
-
-}
-
